"""
Logging configuration module
"""

import logging
from logging.config import dictConfig

from portal.tools import paths


def get_logger_file(name):
    return {
        'level': 'DEBUG',
        'class': 'logging.handlers.RotatingFileHandler',
        'formatter': 'verbose',
        'filename': str(paths.LOG_DIR / f'{name}.log'),
        'maxBytes': 5000000,  # 5MB
        'backupCount': 5
    }


FORMATTERS = {
    'verbose': {
        'format': '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    },

    'simple': {
        'format': '%(levelname)s %(message)s'
    },
    'colored_console': {
        '()': 'coloredlogs.ColoredFormatter',
        'format': "%(asctime)s - %(name)s - %(levelname)s - %(message)s", 'datefmt': '%H:%M:%S'
    },
}

HANDLERS = {
    'console': {
        'level': 'DEBUG',
        'class': 'logging.StreamHandler',
        'formatter': 'colored_console'
    },
    'portal_file': get_logger_file('portal'),
    'access_file': get_logger_file('access'),
    'auth_file': get_logger_file('auth'),
    'storage_file': get_logger_file('storage'),
    'flask_file': get_logger_file('flask')
}

LOGGERS = {
    'portal': {'handlers': ['console', 'portal_file'], 'level': 'INFO', 'propagate': True},
    'portal.auth_log': {
        'handlers': ['console', 'auth_file'], 'level': 'INFO', 'propagate': True
    },
    'portal.access_log': {
        'handlers': ['console', 'access_file'], 'level': 'DEBUG', 'propagate': True
    },
    'tests': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': True},
    'management': {'handlers': ['console'], 'level': 'INFO', 'propagate': True},
    'app': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': True},
    'flask': {'handlers': ['console', 'flask_file'], 'level': 'DEBUG', 'propagate': True},
    'werkzeug': {'handlers': ['console'], 'level': 'DEBUG', 'propagate': True},
    'storage': {'handlers': ['console', 'storage_file'], 'level': 'INFO', 'propagate': True},
}

LOGGING_CONF = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': FORMATTERS,
    'handlers': HANDLERS,
    'loggers': LOGGERS,
}

TRACE_LOG_LVL = 9


def _trace(self, message, *args, **kws):
    if self.isEnabledFor(TRACE_LOG_LVL):
        # Yes, logger takes its '*args' as 'args'.
        self._log(TRACE_LOG_LVL, message, args, **kws)


def add_custom_log_level():
    logging.addLevelName(TRACE_LOG_LVL, 'TRACE')
    logging.Logger.trace = _trace


def load_config(conf_type=None):
    """Loads config based on the config type
    Args:
        conf_type(str): Config type available (dev, test, prod)

    """
    if conf_type == 'test':
        LOGGING_CONF['loggers']['management']['level'] = 'WARNING'
    add_custom_log_level()
    dictConfig(LOGGING_CONF)


def get_logger(*args, **kwargs):
    logger = logging.getLogger(*args, **kwargs)
    return logger


def get_access_logger(*args, **kwargs):
    return logging.getLogger('portal.access_log', *args, **kwargs)


def get_auth_logger(*args, **kwargs):
    return logging.getLogger('portal.auth_log', *args, **kwargs)

ACCESS = get_access_logger()
AUTH = get_auth_logger()
