"""
Emails service
"""

import logging

from emails.template import JinjaTemplate as T
from flask_emails import Message

from portal.database.models import User

log = logging.getLogger(__name__)


class EmailMessage(object):
    def __init__(self, subject: str, body: str, **kwargs):
        """Create email message
        Args:
            subject(str): Subject of the the mail
            body(str): Body of the mail
            **kwargs:
        """
        self._subject = T(subject)
        self._body = T(body)
        self._params = kwargs

    @property
    def subject(self) -> str:
        """Gets subject of a mail
        Returns(str): Subject of the mail
        """
        return self._subject

    @property
    def body(self):
        """Gets body content
        Returns: Email body content
        """
        return self._body

    def create_message(self) -> Message:
        """Creates new message

        Returns(Message): Message instance
        """
        message = Message(
            subject=self.subject,
            html=self.body,
            **self._params
        )
        return message


USERS = dict(
    created=EmailMessage(
        subject="[KONTR] User created: {{username}}",
        body="User {{username}} has been created"
    ),
    deleted=EmailMessage(
        subject="[KONTR] User deleted: {{username}}",
        body="User {{username}} has been deleted"
    ),
    password_update=EmailMessage(
        subject="[KONTR] User's password updated: {{username}}",
        body="User {{username}}'s password has been updated"
    ),
    updated=None
)

COURSES = dict(
    created=None,
    deleted=None,
    updated=None
)

PROJECTS = dict(
    created=None,
    deleted=None,
    updated=None
)

NOTIFICATIONS = dict(
    new=EmailMessage(
        subject="Notification: {{subject}}",
        body="{{content}}"
    )
)

SUBMISSIONS = dict(
    student=EmailMessage(
        subject='[KONTR] Submission evaluation for {{username}}',
        body="{{content}}"
    ),
    teacher=EmailMessage(
        subject='[KONTR] Submission evaluation for {{username}}',
        body="{{content}}"
    )
)

MESSAGES = dict(
    users=USERS,
    courses=COURSES,
    projects=PROJECTS,
    notifications=NOTIFICATIONS,
    submissions=SUBMISSIONS,
)


def get_email_message(collection: str, action: str) -> EmailMessage:
    """Gets email message instance based on templates
    Args:
        collection(str): Collection name
        action(str): Name of the action

    Returns(EmailMessage): Email message instnace
    """
    return MESSAGES[collection][action]


def create_message(collection: str, action: str) -> Message:
    """Creates new message instance
    Args:
        collection(str): Collection name
        action(str): Action name

    Returns(Message): Message instance
    """
    return get_email_message(collection=collection,
                             action=action).create_message()


def send_message_to_user(user: User = None, message: Message = None,
                         context: dict = None) -> Message:
    """Sends message to user
    Args:
        user(User): Receiver
        message(str): Message content
        context(dict): Message context

    Returns(Message): Sent Message instance

    """
    email = user.email
    to = (user.name, email)
    if email and message:
        log.debug(f"[EMAIL] Send message to user [{user.log_name} :: {email}]: {message}")
        message.send(to=to, render=context)
    return message


def notify_user(user: User, collection: str, action: str,
                context: dict = None) -> Message:
    """Notifies the user
    Args:
        user(User): User instance
        collection(str): collection name
        action(str): action name
        context(dict): Message context

    Returns(Message): Sent Message instance
    """
    message = create_message(collection=collection, action=action)
    return send_message_to_user(user=user, message=message, context=context)
